package com.yupi.springbootinit.controller;

import cn.hutool.core.io.FileUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yupi.springbootinit.annotation.AuthCheck;
import com.yupi.springbootinit.common.BaseResponse;
import com.yupi.springbootinit.common.DeleteRequest;
import com.yupi.springbootinit.common.ErrorCode;
import com.yupi.springbootinit.common.ResultUtils;
import com.yupi.springbootinit.constant.CommonConstant;
import com.yupi.springbootinit.constant.FileConstant;
import com.yupi.springbootinit.constant.UserConstant;
import com.yupi.springbootinit.exception.BusinessException;
import com.yupi.springbootinit.exception.ThrowUtils;
import com.yupi.springbootinit.manager.AiManager;
import com.yupi.springbootinit.manager.RedisLimiterManager;
import com.yupi.springbootinit.model.dto.chart.*;
import com.yupi.springbootinit.model.dto.file.UploadFileRequest;
import com.yupi.springbootinit.model.entity.Chart;
import com.yupi.springbootinit.model.entity.User;
import com.yupi.springbootinit.model.enums.FileUploadBizEnum;
import com.yupi.springbootinit.model.vo.BiResponse;
import com.yupi.springbootinit.service.ChartService;
import com.yupi.springbootinit.service.UserService;
import com.yupi.springbootinit.utils.ExcelUtils;
import com.yupi.springbootinit.utils.SqlUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * 图表接口
 *
 * @author <a href="https://github.com/liyupi">程序员鱼皮</a>
 * @from <a href="https://yupi.icu">编程导航知识星球</a>
 */
@RestController
@RequestMapping("/chart")
@Slf4j
public class ChartController {

    @Resource
    private ChartService chartService;

    @Resource
    private UserService userService;

    // region 增删改查

    @Resource
    private AiManager aiManager;

    @Resource
    private RedisLimiterManager redisLimiterManager;

    @Resource
    private ThreadPoolExecutor threadPoolExecutor;

    /**
     * 获取本周每天图表生成成功数
     *
     * @param request
     * @return
     */
    @GetMapping("/week/success/count")
    public BaseResponse<List<Integer>> getWeekChartSuccessCount(HttpServletRequest request) {
        // 获取本周的开始日期（周一）和结束日期（周日）
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date weekStart = calendar.getTime();

        calendar.add(Calendar.DATE, 6);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date weekEnd = calendar.getTime();

        // 创建存储每天成功数量的列表
        List<Integer> dailySuccessCounts = new ArrayList<>();

        // 查询本周每天的成功图表数量
        calendar.setTime(weekStart);
        for (int i = 0; i < 7; i++) {
            Date dayStart = calendar.getTime();

            calendar.set(Calendar.HOUR_OF_DAY, 23);
            calendar.set(Calendar.MINUTE, 59);
            calendar.set(Calendar.SECOND, 59);
            calendar.set(Calendar.MILLISECOND, 999);
            Date dayEnd = calendar.getTime();

            // 查询当天成功的图表数量
            QueryWrapper<Chart> successQuery = new QueryWrapper<>();
            successQuery.in("status", Arrays.asList("succeed", "success"));
            successQuery.between("createTime", dayStart, dayEnd);
            successQuery.eq("isDelete", false);
            long successCount = chartService.count(successQuery);

            dailySuccessCounts.add((int) successCount);

            // 移动到下一天
            calendar.add(Calendar.DATE, 1);
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.set(Calendar.MILLISECOND, 0);
        }

        return ResultUtils.success(dailySuccessCounts);
    }

    /**
     * 获取当前用户本周每天图表生成成功数
     *
     * @param request
     * @return
     */
    @GetMapping("/my/week/success/count")
    public BaseResponse<List<Integer>> getMyWeekChartSuccessCount(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);

        // 获取本周的开始日期（周一）和结束日期（周日）
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date weekStart = calendar.getTime();

        calendar.add(Calendar.DATE, 6);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date weekEnd = calendar.getTime();

        // 创建存储每天成功数量的列表
        List<Integer> dailySuccessCounts = new ArrayList<>();

        // 查询本周每天的成功图表数量
        calendar.setTime(weekStart);
        for (int i = 0; i < 7; i++) {
            Date dayStart = calendar.getTime();

            calendar.set(Calendar.HOUR_OF_DAY, 23);
            calendar.set(Calendar.MINUTE, 59);
            calendar.set(Calendar.SECOND, 59);
            calendar.set(Calendar.MILLISECOND, 999);
            Date dayEnd = calendar.getTime();

            // 查询当天成功的图表数量
            QueryWrapper<Chart> successQuery = new QueryWrapper<>();
            successQuery.in("status", Arrays.asList("succeed", "success"));
            successQuery.between("createTime", dayStart, dayEnd);
            successQuery.eq("userId", loginUser.getId());
            successQuery.eq("isDelete", false);
            long successCount = chartService.count(successQuery);

            dailySuccessCounts.add((int) successCount);

            // 移动到下一天
            calendar.add(Calendar.DATE, 1);
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.set(Calendar.MILLISECOND, 0);
        }

        return ResultUtils.success(dailySuccessCounts);
    }



    /**
     * 获取今日图表生成个数
     *
     * @param request
     * @return
     */
    @GetMapping("/today/count")
    public BaseResponse<Long> getTodayChartCount(HttpServletRequest request) {
        // 获取今天的开始时间（00:00:00）和结束时间（23:59:59）
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date todayStart = calendar.getTime();

        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date todayEnd = calendar.getTime();

        // 创建查询条件
        QueryWrapper<Chart> queryWrapper = new QueryWrapper<>();
        queryWrapper.between("createTime", todayStart, todayEnd);
        queryWrapper.eq("isDelete", false);

        // 查询今日创建的图表数量
        long count = chartService.count(queryWrapper);
        return ResultUtils.success(count);
    }

    /**
     * 获取当前用户今日图表生成个数
     *
     * @param request
     * @return
     */
    @GetMapping("/my/today/count")
    public BaseResponse<Long> getMyTodayChartCount(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);

        // 获取今天的开始时间（00:00:00）和结束时间（23:59:59）
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date todayStart = calendar.getTime();

        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date todayEnd = calendar.getTime();

        // 创建查询条件
        QueryWrapper<Chart> queryWrapper = new QueryWrapper<>();
        queryWrapper.between("createTime", todayStart, todayEnd);
        queryWrapper.eq("userId", loginUser.getId());
        queryWrapper.eq("isDelete", false);

        // 查询当前用户今日创建的图表数量
        long count = chartService.count(queryWrapper);
        return ResultUtils.success(count);
    }


    /**
     * 获取图表生成的成功率
     *
     * @param request
     * @return
     */
    @GetMapping("/gen/success-rate")
   // @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Double> getChartGenerationSuccessRate(HttpServletRequest request) {
        // 只允许管理员访问

        // 查询所有已完成的任务（状态为succeed或failed）
        QueryWrapper<Chart> completedQuery = new QueryWrapper<>();
        completedQuery.in("status", Arrays.asList("succeed", "failed", "success"));
        completedQuery.eq("isDelete", false);
        long completedCount = chartService.count(completedQuery);

        if (completedCount == 0) {
            // 如果没有完成的任务，返回0或null
            return ResultUtils.success(0.0);
        }

        // 查询成功的任务
        QueryWrapper<Chart> successQuery = new QueryWrapper<>();
        successQuery.in("status", Arrays.asList("succeed", "success"));
        successQuery.eq("isDelete", false);
        long successCount = chartService.count(successQuery);

        // 计算成功率
        double successRate = (double) successCount / completedCount;

        return ResultUtils.success(successRate);
    }

    /**
     * 获取当前用户图表生成的成功率
     *
     * @param request
     * @return
     */
    @GetMapping("/my/gen/success-rate")
    public BaseResponse<Double> getMyChartGenerationSuccessRate(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);

        // 查询当前用户所有已完成的任务
        QueryWrapper<Chart> completedQuery = new QueryWrapper<>();
        completedQuery.in("status", Arrays.asList("succeed", "failed", "success"));
        completedQuery.eq("userId", loginUser.getId());
        completedQuery.eq("isDelete", false);
        long completedCount = chartService.count(completedQuery);

        if (completedCount == 0) {
            // 如果没有完成的任务，返回0
            return ResultUtils.success(0.0);
        }

        // 查询当前用户成功的任务
        QueryWrapper<Chart> successQuery = new QueryWrapper<>();
        successQuery.in("status", Arrays.asList("succeed", "success"));
        successQuery.eq("userId", loginUser.getId());
        successQuery.eq("isDelete", false);
        long successCount = chartService.count(successQuery);

        // 计算成功率
        double successRate = (double) successCount / completedCount;

        return ResultUtils.success(successRate);
    }

    /**
     * 获取详细的图表生成统计信息
     *
     * @param request
     * @return
     */
    @GetMapping("/gen/stats")
    //@AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Map<String, Object>> getChartGenerationStats(HttpServletRequest request) {
        // 只允许管理员访问

        Map<String, Object> statsMap = new HashMap<>();

        // 查询总任务数
        QueryWrapper<Chart> totalQuery = new QueryWrapper<>();
        totalQuery.eq("isDelete", false);
        long totalCount = chartService.count(totalQuery);
        statsMap.put("totalCount", totalCount);

        // 查询等待中的任务数
        QueryWrapper<Chart> waitingQuery = new QueryWrapper<>();
        waitingQuery.eq("status", "wait");
        waitingQuery.eq("isDelete", false);
        long waitingCount = chartService.count(waitingQuery);
        statsMap.put("waitingCount", waitingCount);

        // 查询运行中的任务数
        QueryWrapper<Chart> runningQuery = new QueryWrapper<>();
        runningQuery.eq("status", "running");
        runningQuery.eq("isDelete", false);
        long runningCount = chartService.count(runningQuery);
        statsMap.put("runningCount", runningCount);

        // 查询成功的任务数
        QueryWrapper<Chart> successQuery = new QueryWrapper<>();
        successQuery.in("status", Arrays.asList("succeed", "success"));
        successQuery.eq("isDelete", false);
        long successCount = chartService.count(successQuery);
        statsMap.put("successCount", successCount);

        // 查询失败的任务数
        QueryWrapper<Chart> failedQuery = new QueryWrapper<>();
        failedQuery.eq("status", "failed");
        failedQuery.eq("isDelete", false);
        long failedCount = chartService.count(failedQuery);
        statsMap.put("failedCount", failedCount);

        // 计算成功率
        double successRate = totalCount > 0 ? (double) successCount / totalCount : 0;
        statsMap.put("successRate", successRate);

        // 计算已完成任务的成功率
        long completedCount = successCount + failedCount;
        double completedSuccessRate = completedCount > 0 ? (double) successCount / completedCount : 0;
        statsMap.put("completedSuccessRate", completedSuccessRate);

        return ResultUtils.success(statsMap);
    }


    /**
     * 获取图表总数
     *
     * @param request
     * @return
     */
    @GetMapping("/count")
    public BaseResponse<Long> countCharts(HttpServletRequest request) {
        // 只允许管理员访问
        // @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)

        // 创建查询条件
        QueryWrapper<Chart> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("isDelete", false);

        // 如果只需要查询当前用户的图表总数，可以取消注释以下代码
        // User loginUser = userService.getLoginUser(request);
        // queryWrapper.eq("userId", loginUser.getId());

        long count = chartService.count(queryWrapper);
        return ResultUtils.success(count);
    }

    /**
     * 获取当前用户的图表总数
     *
     * @param request
     * @return
     */
    @GetMapping("/my/count")
    public BaseResponse<Long> countMyCharts(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);

        QueryWrapper<Chart> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", loginUser.getId());
        queryWrapper.eq("isDelete", false);

        long count = chartService.count(queryWrapper);
        return ResultUtils.success(count);
    }


    /**
     * 创建
     *
     * @param chartAddRequest
     * @param request
     * @return
     */
    @PostMapping("/add")
    public BaseResponse<Long> addChart(@RequestBody ChartAddRequest chartAddRequest, HttpServletRequest request) {
        if (chartAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Chart chart = new Chart();
        BeanUtils.copyProperties(chartAddRequest, chart);
        User loginUser = userService.getLoginUser(request);
        chart.setUserId(loginUser.getId());
        boolean result = chartService.save(chart);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        long newChartId = chart.getId();
        return ResultUtils.success(newChartId);
    }

    /**
     * 删除
     *
     * @param deleteRequest
     * @param request
     * @return
     */
    @PostMapping("/delete")
    public BaseResponse<Boolean> deleteChart(@RequestBody DeleteRequest deleteRequest, HttpServletRequest request) {
        if (deleteRequest == null || deleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User user = userService.getLoginUser(request);
        long id = deleteRequest.getId();
        // 判断是否存在
        Chart oldChart = chartService.getById(id);
        ThrowUtils.throwIf(oldChart == null, ErrorCode.NOT_FOUND_ERROR);
        // 仅本人或管理员可删除
        if (!oldChart.getUserId().equals(user.getId()) && !userService.isAdmin(request)) {
            throw new BusinessException(ErrorCode.NO_AUTH_ERROR);
        }
        boolean b = chartService.removeById(id);
        return ResultUtils.success(b);
    }

    /**
     * 更新（仅管理员）
     *
     * @param chartUpdateRequest
     * @return
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateChart(@RequestBody ChartUpdateRequest chartUpdateRequest) {
        if (chartUpdateRequest == null || chartUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Chart chart = new Chart();
        BeanUtils.copyProperties(chartUpdateRequest, chart);
        long id = chartUpdateRequest.getId();
        // 判断是否存在
        Chart oldChart = chartService.getById(id);
        ThrowUtils.throwIf(oldChart == null, ErrorCode.NOT_FOUND_ERROR);
        boolean result = chartService.updateById(chart);
        return ResultUtils.success(result);
    }

    /**
     * 根据 id 获取
     *
     * @param id
     * @return
     */
    @GetMapping("/get")
    public BaseResponse<Chart> getChartById(long id, HttpServletRequest request) {
        if (id <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Chart chart = chartService.getById(id);
        if (chart == null) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR);
        }
        return ResultUtils.success(chart);
    }

    /**
     * 分页获取列表（封装类）
     *
     * @param chartQueryRequest
     * @param request
     * @return
     */
    @PostMapping("/list/page")
    public BaseResponse<Page<Chart>> listChartByPage(@RequestBody ChartQueryRequest chartQueryRequest,
            HttpServletRequest request) {
        long current = chartQueryRequest.getCurrent();
        long size = chartQueryRequest.getPageSize();
        // 限制爬虫
        ThrowUtils.throwIf(size > 20, ErrorCode.PARAMS_ERROR);
        Page<Chart> chartPage = chartService.page(new Page<>(current, size),
                getQueryWrapper(chartQueryRequest));
        return ResultUtils.success(chartPage);
    }

    /**
     * 分页获取当前用户创建的资源列表
     *
     * @param chartQueryRequest
     * @param request
     * @return
     */
    @PostMapping("/my/list/page")
    public BaseResponse<Page<Chart>> listMyChartByPage(@RequestBody ChartQueryRequest chartQueryRequest,
            HttpServletRequest request) {
        if (chartQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User loginUser = userService.getLoginUser(request);
        chartQueryRequest.setUserId(loginUser.getId());
        long current = chartQueryRequest.getCurrent();
        long size = chartQueryRequest.getPageSize();
        // 限制爬虫
        ThrowUtils.throwIf(size > 20, ErrorCode.PARAMS_ERROR);
        Page<Chart> chartPage = chartService.page(new Page<>(current, size),
                getQueryWrapper(chartQueryRequest));
        return ResultUtils.success(chartPage);
    }

    // endregion

    /**
     * 编辑（用户）
     *
     * @param chartEditRequest
     * @param request
     * @return
     */
    @PostMapping("/edit")
    public BaseResponse<Boolean> editChart(@RequestBody ChartEditRequest chartEditRequest, HttpServletRequest request) {
        if (chartEditRequest == null || chartEditRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Chart chart = new Chart();
        BeanUtils.copyProperties(chartEditRequest, chart);
        User loginUser = userService.getLoginUser(request);
        long id = chartEditRequest.getId();
        // 判断是否存在
        Chart oldChart = chartService.getById(id);
        ThrowUtils.throwIf(oldChart == null, ErrorCode.NOT_FOUND_ERROR);
        // 仅本人或管理员可编辑
        if (!oldChart.getUserId().equals(loginUser.getId()) && !userService.isAdmin(loginUser)) {
            throw new BusinessException(ErrorCode.NO_AUTH_ERROR);
        }
        boolean result = chartService.updateById(chart);
        return ResultUtils.success(result);
    }



    /**
     * 智能分析（同步）
     *
     * @param multipartFile
     * @param genChartByAiRequest
     * @param request
     * @return
     */
    @PostMapping("/gen")
    public BaseResponse<BiResponse> genChartByAi(@RequestPart("file") MultipartFile multipartFile,
                                                 GenChartByAiRequest genChartByAiRequest, HttpServletRequest request) {
        String name = genChartByAiRequest.getName();
        String goal = genChartByAiRequest.getGoal();
        String chartType = genChartByAiRequest.getChartType();
        // 校验
        ThrowUtils.throwIf(StringUtils.isBlank(goal), ErrorCode.PARAMS_ERROR, "目标为空");
        ThrowUtils.throwIf(StringUtils.isNotBlank(name) && name.length() > 100, ErrorCode.PARAMS_ERROR, "名称过长");

        //校验文件
        long size =multipartFile.getSize();
        String originalFilename = multipartFile.getOriginalFilename();
        //校验文件大小
        final long ONE_MB = 10 *1024*1024L;
        ThrowUtils.throwIf(size>ONE_MB,ErrorCode.PARAMS_ERROR,"文件超过10M");
        //校验文件后缀
        String suffix = FileUtil.getSuffix(originalFilename);
        final  List<String> validFileSuffixList = Arrays.asList("xlsx","xls");
        ThrowUtils.throwIf(!validFileSuffixList.contains(suffix),ErrorCode.PARAMS_ERROR,"文件后缀不合法");



//        final String prompt = "你是一个数据分析师和前端开发专家，接下来我会按照以下固定格式给你提供内容：\n" +
//                "分析需求：\n" +
//                "{数据分析的需求或者目标}\n" +
//                "原始数据：\n" +
//                "{csv格式的原始数据，用,作为分隔符}\n" +
//                "请根据这两部分内容，按照以下指定格式生成内容（此外不要输出任何多余的开头、结尾、注释）\n" +
//                "'【【【【'\n" +
//                "{前端 Echarts V5 的 option 配置对象js代码，合理地将数据进行可视化，不要生成任何多余的内容，比如注释}\n" +
//                "'【【【【'\n" +
//                "{明确的数据分析结论、越详细越好，不要生成多余的注释}";

        User loginUser = userService.getLoginUser(request);
        // 限流判断，每个用户一个限流器
        redisLimiterManager.doRateLimit("genChartByAi_" + loginUser.getId());


        // 分析需求：
        // 分析网站用户的增长情况
        // 原始数据：
        // 日期,用户数
        // 1号,10
        // 2号,20
        // 3号,30

        // 构造用户输入
        StringBuilder userInput = new StringBuilder();
        userInput.append("分析需求：").append("\n");

        // 拼接分析目标
        String userGoal = goal;
        if (StringUtils.isNotBlank(chartType)) {
            userGoal += "，请使用" + chartType;
        }
        userInput.append(userGoal).append("\n");
        userInput.append("原始数据：").append("\n");
        // 压缩后的数据
        String csvData = ExcelUtils.excelToCsv(multipartFile);
        userInput.append(csvData).append("\n");

        String result = aiManager.sendMsgToXingHuo(true,userInput.toString());


        String[] splits = result.split("'【【【【'");
        if (splits.length < 3) {
            throw new BusinessException(ErrorCode.SYSTEM_ERROR, "AI 生成错误");
        }

        String genChart = splits[1].trim();
        String genResult = splits[2].trim();


        // 插入到数据库
        Chart chart = new Chart();
        chart.setName(name);
        chart.setGoal(goal);
        chart.setChartData(csvData);
        chart.setChartType(chartType);
        chart.setGenChart(genChart);
        chart.setGenResult(genResult);
        chart.setUserId(loginUser.getId());
        boolean saveResult = chartService.save(chart);
        ThrowUtils.throwIf(!saveResult, ErrorCode.SYSTEM_ERROR, "图表保存失败");

        BiResponse biResponse = new BiResponse();
        biResponse.setGenChart(genChart);
        biResponse.setGenResult(genResult);
        biResponse.setChartId(chart.getId());
        return ResultUtils.success(biResponse);

//        // 校验文件
//        long size = multipartFile.getSize();
//        String originalFilename = multipartFile.getOriginalFilename();
//        // 校验文件大小
//        final long ONE_MB = 1024 * 1024L;
//        ThrowUtils.throwIf(size > ONE_MB, ErrorCode.PARAMS_ERROR, "文件超过 1M");
//        // 校验文件后缀 aaa.png
//        String suffix = FileUtil.getSuffix(originalFilename);
//        final List<String> validFileSuffixList = Arrays.asList("xlsx");
//        ThrowUtils.throwIf(!validFileSuffixList.contains(suffix), ErrorCode.PARAMS_ERROR, "文件后缀非法");
//
//        User loginUser = userService.getLoginUser(request);
//
//
//        return ResultUtils.success("");
    }


    /**
     * 智能分析（异步）
     *
     * @param multipartFile
     * @param genChartByAiRequest
     * @param request
     * @return
     */
    @PostMapping("/gen/async")
    public BaseResponse<BiResponse> genChartByAiAsync(@RequestPart("file") MultipartFile multipartFile,
                                                      GenChartByAiRequest genChartByAiRequest, HttpServletRequest request) {
        String name = genChartByAiRequest.getName();
        String goal = genChartByAiRequest.getGoal();
        String chartType = genChartByAiRequest.getChartType();
        // 校验
        ThrowUtils.throwIf(StringUtils.isBlank(goal), ErrorCode.PARAMS_ERROR, "目标为空");
        ThrowUtils.throwIf(StringUtils.isNotBlank(name) && name.length() > 100, ErrorCode.PARAMS_ERROR, "名称过长");
        // 校验文件
        long size = multipartFile.getSize();
        String originalFilename = multipartFile.getOriginalFilename();
        // 校验文件大小
        final long ONE_MB = 1024 * 1024L;
        ThrowUtils.throwIf(size > ONE_MB, ErrorCode.PARAMS_ERROR, "文件超过 1M");
        // 校验文件后缀 aaa.png
        String suffix = FileUtil.getSuffix(originalFilename);
        final List<String> validFileSuffixList = Arrays.asList("xlsx", "xls");
        ThrowUtils.throwIf(!validFileSuffixList.contains(suffix), ErrorCode.PARAMS_ERROR, "文件后缀非法");

        User loginUser = userService.getLoginUser(request);
        // 限流判断，每个用户一个限流器
        redisLimiterManager.doRateLimit("genChartByAi_" + loginUser.getId());


        // 构造用户输入
        StringBuilder userInput = new StringBuilder();
        userInput.append("分析需求：").append("\n");

        // 拼接分析目标
        String userGoal = goal;
        if (StringUtils.isNotBlank(chartType)) {
            userGoal += "，请使用" + chartType;
        }
        userInput.append(userGoal).append("\n");
        userInput.append("原始数据：").append("\n");
        // 压缩后的数据
        String csvData = ExcelUtils.excelToCsv(multipartFile);
        userInput.append(csvData).append("\n");

        // 插入到数据库
        Chart chart = new Chart();
        chart.setName(name);
        chart.setGoal(goal);
        chart.setChartData(csvData);
        chart.setChartType(chartType);
        chart.setStatus("wait");
        chart.setUserId(loginUser.getId());
        boolean saveResult = chartService.save(chart);  // 直接插入
        ThrowUtils.throwIf(!saveResult, ErrorCode.SYSTEM_ERROR, "图表保存失败");

        // todo 建议处理任务队列满了后，抛异常的情况
        try{
            CompletableFuture.runAsync(() -> {
                // 先修改图表任务状态为 “执行中”。等执行成功后，修改为 “已完成”、保存执行结果；执行失败后，状态修改为 “失败”，记录任务失败信息。
                Chart updateChart = new Chart();
                updateChart.setId(chart.getId());
                updateChart.setStatus("running");
                boolean b = chartService.updateById(updateChart);
                if (!b) {
                    handleChartUpdateError(chart.getId(), "更新图表执行中状态失败");
                    return;
                }
                // 调用 AI
                String result = aiManager.sendMsgToXingHuo(true,userInput.toString());
                String[] splits = result.split("'【【【【'");
                if (splits.length < 3) {
                    handleChartUpdateError(chart.getId(), "AI 生成错误");
                    return;
                }
                String genChart = splits[1].trim();
                String genResult = splits[2].trim();
                Chart updateChartResult = new Chart();
                updateChartResult.setId(chart.getId());
                updateChartResult.setGenChart(genChart);
                updateChartResult.setGenResult(genResult);
                // todo 建议定义状态为枚举值
                updateChartResult.setStatus("succeed");
                boolean updateResult = chartService.updateById(updateChartResult);
                if (!updateResult) {
                    handleChartUpdateError(chart.getId(), "更新图表成功状态失败");
                }
            }, threadPoolExecutor);
        }catch (RejectedExecutionException e) {
            log.error("任务队列已满，无法提交异步任务", e);
            // 更新图表状态为失败
            handleChartUpdateError(chart.getId(), "任务队列已满，无法处理");
        }


        BiResponse biResponse = new BiResponse();
        biResponse.setChartId(chart.getId());
        return ResultUtils.success(biResponse);
    }



    //工具类
    private void handleChartUpdateError(long chartId, String execMessage) {
        Chart updateChartResult = new Chart();
        updateChartResult.setId(chartId);
        updateChartResult.setStatus("failed");
        updateChartResult.setExecMessage("execMessage");
        boolean updateResult = chartService.updateById(updateChartResult);
        if (!updateResult) {
            log.error("更新图表失败状态失败" + chartId + "," + execMessage);
        }
    }


//    /**
//     * 智能分析
//     *
//     * @param multipartFile
//     * @param genChartByAiRequest
//     * @param request
//     * @return
//     */
//    @PostMapping("/gen")
//    public BaseResponse<BiResponse> genChartByAi(@RequestPart("file") MultipartFile multipartFile,
//                                             GenChartByAiRequest genChartByAiRequest, HttpServletRequest request) {
//        String name = genChartByAiRequest.getName();
//        String goal = genChartByAiRequest.getGoal();
//        String chartType = genChartByAiRequest.getChartType();
//        //校验
//        ThrowUtils.throwIf(StringUtils.isBlank(goal),ErrorCode.PARAMS_ERROR,"目标为空");
//        ThrowUtils.throwIf(StringUtils.isNotBlank(name) && name.length()>100,ErrorCode.PARAMS_ERROR,"名称过长");
//
//        //校验文件
//        long size =multipartFile.getSize();
//        String originalFilename = multipartFile.getOriginalFilename();
//        //校验文件大小
//        final long ONE_MB = 10 *1024*1024L;
//        ThrowUtils.throwIf(size>ONE_MB,ErrorCode.PARAMS_ERROR,"文件超过10M");
//        //校验文件后缀
//        String suffix = FileUtil.getSuffix(originalFilename);
//        final  List<String> validFileSuffixList = Arrays.asList("xlsx","xls");
//        ThrowUtils.throwIf(!validFileSuffixList.contains(suffix),ErrorCode.PARAMS_ERROR,"文件后缀不合法");
//
//        //获取用户信息
//        User loginUser = userService.getLoginUser(request);
//        //限流判断，每个用户一个限流器(方法名+用户id)
//        redisLimiterManager.doRateLimit("genChartByAi_"+loginUser.getId());
//
//
//        //系统预设
////        final String prompt = "你是一个数据分析师和前端开发专家，接下来我会按照以下固定格式给你提供内容：\n" +
////                "分析需求：\n" +
////                "(数据分析的需求或者目标}\n" +
////                "原始数据：\n" +
////                "{csv格式的原始数据，用，作为分隔符]\n" +
////                "请根据这两部分内容，按照以下指定格式生成内容（此外不要输出任何多余的开头、结尾、注释）\n" +
////                "'【【【【'\n" +
////                "{前端Echarts V5的option配置对象js代码，合理地将数据进行可视化，不要生成任何多余的内容，比如注释}\n" +
////                "'【【【【'\n" +
////                "(明确的数据分析结论、越详细越好，不要生成多余的注释}";
//
//
//        //构造用户输入
//        StringBuilder userInput = new StringBuilder();
//        userInput.append("分析需求：").append("\n");
//
//        //拼接分析目标
//        String userGoal =goal;
//        if(StringUtils.isNotBlank(chartType)){
//            userGoal+=",请使用"+chartType;
//        }
//        userInput.append(userGoal).append("\n");
//        userInput.append("原始数据：").append("\n");
//
//        //压缩后的数据
//        String csvData=ExcelUtils.excelToCsv(multipartFile);
//        userInput.append(csvData).append("\n");
//
//        //插入到数据库中
//        Chart chart = new Chart();
//        chart.setName(name);
//        chart.setGoal(goal);
//        chart.setChartData(csvData);
//        chart.setChartType(chartType);
//        chart.setStatus("wait");
//        chart.setUserId(loginUser.getId());
//        boolean saveResult = chartService.save(chart);
//        ThrowUtils.throwIf(!saveResult,ErrorCode.SYSTEM_ERROR,"图表保存失败");
//
//        //处理任务队列满了后，抛异常的情况
//        try {
//            CompletableFuture.runAsync(() -> {
//                // 先修改图表任务状态为 “执行中”。等执行成功后，修改为 “已完成”、保存执行结果；执行失败后，状态修改为 “失败”，记录任务失败信息。
//                Chart updateChart = new Chart();
//                updateChart.setId(chart.getId());
//                updateChart.setStatus("running");
//                boolean b = chartService.updateById(updateChart);
//                if (!b) {
//                    handleChartUpdateError(chart.getId(), "更新图表执行中状态失败");
//                    return;
//                }
//                //调用ai
//                String result = aiManager.sendMsgToXingHuo(true,userInput.toString());
//                String[] splits = result.split("'【【【【'");
//                if(splits.length<3){
//                    handleChartUpdateError(chart.getId(), "AI 生成错误");
//                    return;
//                }
//                String genChart =splits[1].trim();
//                String genResult = splits[2].trim();
//                Chart updateChartResult = new Chart();
//                updateChartResult.setId(chart.getId());
//                updateChartResult.setGenChart(genChart);
//                updateChartResult.setGenResult(genResult);
//                updateChartResult.setStatus("success");
//                boolean updateResult = chartService.updateById(updateChartResult);
//                if (!updateResult) {
//                    handleChartUpdateError(chart.getId(), "更新图表成功状态失败");
//                }
//            },threadPoolExecutor);
//        }catch (RejectedExecutionException e) {
//            System.err.println("任务被拒绝，线程池任务队列已满: " + e.getMessage());
//        }
//
//        BiResponse biResponse =new BiResponse();
//        biResponse.setChartId(chart.getId());
//        return  ResultUtils.success(biResponse);
//
//    }
//
//
//    //工具类
//    private void handleChartUpdateError(long chartId, String execMessage) {
//        Chart updateChartResult = new Chart();
//        updateChartResult.setId(chartId);
//        updateChartResult.setStatus("failed");
//        updateChartResult.setExecMessage("execMessage");
//        boolean updateResult = chartService.updateById(updateChartResult);
//        if (!updateResult) {
//            log.error("更新图表失败状态失败" + chartId + "," + execMessage);
//        }
//    }



    /**
     * 获取查询包装类
     *
     * @param chartQueryRequest
     * @return
     */
    private QueryWrapper<Chart> getQueryWrapper(ChartQueryRequest chartQueryRequest) {
        QueryWrapper<Chart> queryWrapper = new QueryWrapper<>();
        if (chartQueryRequest == null) {
            return queryWrapper;
        }
        Long id = chartQueryRequest.getId();
        String name = chartQueryRequest.getName();
        String goal = chartQueryRequest.getGoal();
        String chartType = chartQueryRequest.getChartType();
        Long userId = chartQueryRequest.getUserId();
        String sortField = chartQueryRequest.getSortField();
        String sortOrder = chartQueryRequest.getSortOrder();

        queryWrapper.eq(id!=null && id>0,"id",id);
        queryWrapper.like(StringUtils.isNotBlank(name),"name",name);
        queryWrapper.eq(StringUtils.isNotBlank(goal),"goal",goal);
        queryWrapper.eq(StringUtils.isNotBlank(chartType),"chartType",chartType);
        queryWrapper.eq(ObjectUtils.isNotEmpty(userId), "userId", userId);
        queryWrapper.eq("isDelete",false);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }

}
